// *****************************************************************
// AS-megaM.c
// AS-kit
//  : 15.04.2019
// *****************************************************************

#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "Kernel.h"
#include "InternalUSART.h"
#include "ADC.h"
#include "DAC.h"
#include "LCD.h"

XTAL_FREQ xtal_freq;
char szXTALfreq[15];
bool connected = false;
dword seconds = 0;

void Port_Init(void)
{
	//  
	PORTA = 0xFF;
	PORTB = 0xFF;
	PORTC = 0xFF;
	PORTD = 0xFF;
	PORTE = 0xFF;
	PORTF = 0xFF;
	LED_DDR |= LED1 | LED2;
	LED_PORT &= ~(LED1 | LED2);
}
 
  //  
void Timers_Start(void)
{
	TIFR  |= (1<<TOV0) | (1<<OCF0);
	ASSR  |= (1<<AS0);
	TIMSK = 0x00;
	TCNT0 = 0;
	
	TIFR  |= (1<<TOV1) | (1<<OCF1A) | (1<<OCF1B) | (1<<OCF1C);
	TIMSK &= ~((1<<TOIE1)|(1<<OCIE1A)|(1<<OCIE1B));
	ETIMSK &= ~(1<<OCIE1C);
	TCCR1B = 0b00000000;
	TCNT1  = 0;
	TCCR1A = 0;
	TCCR0 = 3;
	TCCR1B = 3;
	TCNT0 = 0;
	TCNT1 = 0;
}

void Timer0_Start(void)
{
	TIFR  |= (1<<TOV0) | (1<<OCF0);
	ASSR  |= (1<<AS0);
	TIMSK &= ~(1<<OCIE0);
	TIMSK |= (1<<TOIE0);
	TCNT0 = 0;
	TCCR0 = 1;
}

void Timer1_Stop(void)
{
	TCCR1B = 0;
}

//       
void GetXTALfreq(void)
{
	float XTALfreq;
	unsigned int result;
	Timers_Start();
	for (int i=0; i<2; i++)
	{
		while (TCNT0 == 0)
		;
		while (TCNT0 != 0)
		;
		result = TCNT1;
		TCNT1 = 0;
	}
	XTALfreq = result * 0.000256;
	dtostrf(XTALfreq,4,2,szXTALfreq);
	strcat(szXTALfreq, " MHz");
	Timer1_Stop();


 //       
	if (abs(XTALfreq - 7.37) < 0.1)
	xtal_freq = XTAL_7372800;
  else if (abs(XTALfreq - 8) < 0.4)
  xtal_freq = XTAL_8000000;
	else if (abs(XTALfreq - 11.06) < 0.1)
	xtal_freq = XTAL_11059200;
	else if (abs(XTALfreq - 14.74) < 0.1)
	xtal_freq = XTAL_14745600;
	else if (abs(XTALfreq - 16) < 0.4)
	xtal_freq = XTAL_16000000;
	
}
 //     ATmega128      AS-mega Demo
void XTAL_WriteFreq(void)
{
	USART1_WriteString(szXTALfreq);
}

void Ports_WriteStatus(void)
{
	USART1_Write(PORTB);
	USART1_Write(PORTC);
	USART1_Write(PORTD);
	USART1_Write(PORTE);
	USART1_Write(DDRB);
	USART1_Write(DDRC);
	USART1_Write(DDRD);
	USART1_Write(DDRE);
	USART1_Write(PINB);
	USART1_Write(PINC);
	USART1_Write(PIND);
	USART1_Write(PINE);
}

void Ports_ReadStatus(void)
{
	PORTB = USART1_Read();
	PORTC = USART1_Read();
	PORTD = USART1_Read();
	PORTE = USART1_Read();
	DDRB = USART1_Read();
	DDRC = USART1_Read();
	DDRD = USART1_Read();
	DDRE = USART1_Read();
	PINB = USART1_Read();
	PINC = USART1_Read();
	PIND = USART1_Read();
	PINE = USART1_Read();
}

 //          AS-mega Demo
 //  4   ATmega128    JTAG,
 //    ,   fuse- JTAGEN  ""
void ADC_WriteValues(void)
{
	unsigned char i;
	unsigned int value;
	for (i=0; i<8; i++)
	{
		ADC_Init(i, ADC_PRESCALER_64);
		value = ADC_GetValue();
		USART1_WriteWord(value);
	};
	
}

void ADC_ReadRef(void)
{
	unsigned char data;
	data = USART1_Read();
	if (data == ADC_AREF)
	ADMUX &= ~(1<<REFS1 | 1<<REFS0);
	else if (data == ADC_VREF)
	ADMUX = (ADMUX | 1<<REFS0) & ~(1<<REFS1);
	else if (data == ADC_INTREF)
	ADMUX |= (1<<REFS1 | 1<<REFS0);
	
}

 //     
static volatile bool DAC_on;

void DAC_Off(void)
{
	DAC_on = false;
}

void DAC_SetConstValue(void)
{
	byte data;
	data = USART1_Read();
	DAC_WriteA(data);
}

void DAC_SetSawOperation(void)
{
	DAC_on = true;

	byte minimum, maximum;
	minimum = USART1_Read();
	maximum = USART1_Read();

	sei();

	byte i;
	while (true)
	{
		for (i = minimum; i < maximum; i++)
		DAC_WriteA(i);
		if (!DAC_on)
		break;
	}
	DAC_WriteA(0x00);
}

void DAC_SetTriangleOperation(void)
{
	DAC_on = true;

	byte minimum, maximum;
	minimum = USART1_Read();
	maximum = USART1_Read();

	sei();

	byte i;
	while (true)
	{
		for (i = minimum; i < maximum; i++)
		DAC_WriteA(i);
		for (i = maximum; i > minimum; i--)
		DAC_WriteA(i);
		if (!DAC_on)
		break;
	}
	DAC_WriteA(0x00);
}

void DAC_SetMeandreOperation(void)
{
	DAC_on = true;

	byte minimum, maximum;
	minimum = USART1_Read();
	maximum = USART1_Read();

	sei();

	while (true)
	{
		DAC_WriteA(minimum);
		DAC_WriteA(maximum);
		if (!DAC_on)
		break;
	}
	DAC_WriteA(0x00);
}

 //    AS-mega Demo       ""
void LCD_ReadText(void)
{
	unsigned char data;

	LCD_SetPos(0,0);
	LCD_WriteString("                    ");
	LCD_SetPos(0,0);
	data = USART1_Read();
	while (data != '\n')
	{
		LCD_WriteChar(data);
		data = USART1_Read();
	}
	LCD_SetPos(0,1);
	LCD_WriteString("                    ");
	LCD_SetPos(0,1);
	data = USART1_Read();
	while (data != '\n')
	{
		LCD_WriteChar(data);
		data = USART1_Read();
	}
	LCD_Refresh();
}

 //    AS-mega Demo     
void Time_ReadValue(void)
{
	seconds = (dword)USART1_Read() << 24;
	seconds |= (dword)USART1_Read() << 16;
	seconds |= (dword)USART1_Read() << 8;
	seconds |= (dword)USART1_Read();
}


 //  
void LEDs_Flip(void)
{
	static bool led1 = false;
	if (led1 == false)
	{
		led1 = true;
		LED_PORT |= LED1;
		LED_PORT &= ~LED2;
	}
	else
	{
		led1 = false;
		LED_PORT &= ~LED1;
		LED_PORT |= LED2;
	}

}

int main(void)
{

 //   USART ATmega128,   19200 
	cli();
	Port_Init();
	GetXTALfreq();
	Timer0_Start();
	USART1_Open(xtal_freq, BR_19200, CHAR_SIZE_8BIT, PARITY_DISABLED, ONE_STOP_BIT);
	DAC_Open();
	Delay(1000);
	LCD_Init();

	// External interrupt init
	EICRB = 0b10101010;
	EIMSK = 0b11110000;

	// ADC initial reference init
	ADMUX = (ADMUX | 1<<REFS0) & ~(1<<REFS1);
	
	
	//      
	Delay(500);
	//  
	LCD_ClearBuf();
	LCD_WriteString("");
	LCD_Refresh();
	Delay(1000);

	LCD_Init();
	Delay(500);
	//   
	LCD_ClearBuf();
	LCD_WriteString(" *** AS-Kit *** ");
	LCD_Refresh();
	Delay(2000);


	//      
	LCD_ClearBuf();
	LCD_WriteString("XTAL ");
	LCD_WriteString(szXTALfreq);
	LCD_Refresh();
	Delay(2000);

	LCD_ClearBuf();
	LCD_Refresh();

	//   
	sei();
	
	while(true);

	return 0;
}

ISR(TIMER0_OVF_vect)  
{
	static unsigned char i = 0;
	static unsigned char j = 0;
	bool refresh = false;
	
	i++;
	j++;
	
	if (j == 64)
	{
		j = 0;
		refresh = true;
	}

	if (i == 128)
	{
		i = 0;
		seconds++;
	}

	if (!connected && refresh == true)
	{
		refresh = false;
		LEDs_Flip();
		LCD_SetPos(0,0);
		LCD_WriteTimeHHMMSS(seconds);
		LCD_Refresh();
	}
} 

ISR(INT4_vect) 
{
	LCD_SetPos(0,1);
	LCD_WriteString("                    ");
	LCD_SetPos(0,1);
	LCD_WriteString("Button 1");
	LCD_Refresh();
}

ISR(INT5_vect) 
{
	LCD_SetPos(0,1);
	LCD_WriteString("                    ");
	LCD_SetPos(0,1);
	LCD_WriteString("Button 2");
	LCD_Refresh();
	DAC_WriteA(0xFF);
}

ISR(INT6_vect)  
{
	LCD_SetPos(0,1);
	LCD_WriteString("                    ");
	LCD_SetPos(0,1);
	LCD_WriteString("Button 3");
	LCD_Refresh();
}

ISR(INT7_vect)  
{
	LCD_SetPos(0,1);
	LCD_WriteString("                    ");
	LCD_SetPos(0,1);
	LCD_WriteString("Button 4");
	LCD_Refresh();
}

 //    ,     
 //        ATmega128
 //     "  "   AS-mega Demo
ISR(USART1_RX_vect)  
{
	unsigned char data;
	data = USART1_Read();
	if (!connected)
	connected = true;
	
	if (data == CMD_LED)
	{
		data = USART1_Read();
		if (data == LED1_ON)
		LED_PORT |= LED1;
		else if (data == LED1_OFF)
		LED_PORT &= ~LED1;
		else if (data == LED2_ON)
		LED_PORT |= LED2;
		else if (data == LED2_OFF)
		LED_PORT &= ~LED2;
	}
	else if (data == '\r')
	USART1_WriteString("\n\r");
	else if(data == CMD_PING)
	{
		if (!connected)
		LED_PORT |= (LED1|LED2);
		connected = true;
		
		USART1_WriteString("AS-MegaM. Firmware v. 2.0");
		USART1_Write('\0');
	}
	else if(data == CMD_EXIT)
	{
		DAC_Off();
		connected = false;
	}
	else if(data == CMD_XTAL)
	{
		USART1_WriteString(szXTALfreq);
		USART1_Write('\0');
	}
	else if(data == CMD_PORTS)
	{
		data = USART1_Read();
		if (data == CMD_PORT_READ)
		Ports_WriteStatus();
		else if (data == CMD_PORT_WRITE)
		Ports_ReadStatus();
	}
	else if(data == CMD_ADC)
	{
		data = USART1_Read();
		if (data == CMD_ADC_READ)
		ADC_WriteValues();
		else if (data == CMD_ADC_SETREF)
		ADC_ReadRef();
	}
	else if(data == CMD_DAC)
	{
		data = USART1_Read();
		if (data == DAC_OFF)
		DAC_Off();
		else if (data == DAC_VOLTAGE)
		DAC_SetConstValue();
		else if (data == DAC_SAW)
		DAC_SetSawOperation();
		else if (data == DAC_TRIANGLE)
		DAC_SetTriangleOperation();
		else if (data == DAC_MEANDRE)
		DAC_SetMeandreOperation();
	}
	else if(data == CMD_LCD)
	{
		LCD_ReadText();
	}
	else if(data == CMD_TIME)
	{
		Time_ReadValue();
	}
	else
	USART1_Write(data);
}
