
// *****************************************************************
// Dual USART driver
// AS-kit
//  : 15.04.2019
// *****************************************************************



#include "Kernel.h"
#include "InternalUSART.h"

unsigned int _ubrr[5][10] = {	//    USART    
	{ 191,95,47,31,23,15,11,7,5,3 },	  //   7.372 MHz
	{ 207,103,51,34,25,16,12,8,6,3 },	  //   8.000 MHz
	{ 287,143,71,47,35,23,17,11,8,5},	  //  11.059 MHz
	{ 383,191,95,63,47,31,23,15,11,7 },	//  14.745 MHz
	{ 416,207,103,68,51,34,25,16,12,8 }	//  16.000 MHz
						    };                    //  U2X = 0 

// USART 0

void USART0_Open(
	XTAL_FREQ xtalFreq,
	BAUD_RATE BaudRate,
	CHAR_SIZE CharSize,
	PARITY Parity,
	STOP_BITS StopBits)
{
	unsigned int baudrate = _ubrr[xtalFreq][BaudRate];
    
	DDRE = (DDRE & 0b11111110) | 0b00000010;
	PORTE |= 0b00000011;

	UCSR0A = 0;
	UBRR0H = (baudrate>>8)&0xff;
	UBRR0L = (baudrate)&0xff;
	UCSR0C = (StopBits << USBS0) | (Parity << UPM00) | ((CharSize&(0b00000011)) << UCSZ00);
	UCSR0B = (1<<RXCIE0) | (1<<RXEN0) | (1<<TXEN0) | ((CharSize&0b00000100) << UCSZ02);
}

void USART0_Close()
{
	DDRE &= DDRE & 0b11111100;
	PORTE |= 0b00000011;
}

void USART0_Write(unsigned char b)
{
	while(!(UCSR0A & (1<<UDRE0)))
		;
	UDR0 = b;
}

void USART0_WriteWord(unsigned int data)
{
	USART0_Write(data>>8);
	USART0_Write(data);
}

void USART0_WriteDWord(unsigned long data)
{
	USART0_Write(data>>24);
	USART0_Write(data>>16);
	USART0_Write(data>>8);
	USART0_Write(data);
}

void USART0_WriteString(char* string)
{
	while (*string)
		USART0_Write(*string++);
}

void USART0_WriteInt(unsigned int value)
{
	if(value >= 10000)
		USART0_Write(((value/10000)%10)+'0');
	if(value >= 1000)
		USART0_Write(((value/1000)%10)+'0');
	if(value >= 100)
		USART0_Write(((value/100)%10)+'0');
	if(value >= 10)
		USART0_Write(((value/10)%10)+'0');
	USART0_Write((value%10)+'0');
}

unsigned char USART0_Read(void)
{
	while(!(UCSR0A & (1<<RXC0)))
		;
	return UDR0;
}

unsigned char USART0_IsData(void)
{
	if((UCSR0A & (1<<RXC0)))
		return true;
	else
		return false;
}

void USART0_Purge(void)
{
	UCSR0A |= (1<<RXC0);
}

void USART0_Flush(void)
{
}

// USART 1

void USART1_Open(
	XTAL_FREQ xtalFreq,
	BAUD_RATE BaudRate,
	CHAR_SIZE CharSize,
	PARITY Parity,
	STOP_BITS StopBits)
{
	unsigned int baudrate = _ubrr[xtalFreq][BaudRate];

	DDRD = ((DDRD & 0b11111011) | 0b00001000);
	PORTD |= 0b00001100;

	UCSR1A = 0;
	UBRR1H = (baudrate>>8)&0xff;
	UBRR1L = (baudrate)&0xff;
	UCSR1C = (StopBits << USBS0) | (Parity << UPM00) | ((CharSize&(0b00000011)) << UCSZ00);
	UCSR1B = (1<<RXCIE0) | (1<<RXEN0) | (1<<TXEN0) | ((CharSize&0b00000100) << UCSZ02);
}

void USART1_Close()
{
	DDRD &= DDRD & 0b11110011;
	PORTD |= 0b00001100;
}

void USART1_Write(unsigned char b)
{
	while(!(UCSR1A & (1<<UDRE1)))
		;
	UDR1 = b;
}

void USART1_WriteWord(unsigned int data)
{
	USART1_Write(data>>8);
	USART1_Write(data);
}

void USART1_WriteDWord(unsigned long data)
{
	USART1_Write(data>>24);
	USART1_Write(data>>16);
	USART1_Write(data>>8);
	USART1_Write(data);
}

void USART1_WriteString(char* string)
{
	while (*string)
		USART1_Write(*string++);
}

void USART1_WriteInt(unsigned int value)
{
	if(value >= 10000)
		USART1_Write(((value/10000)%10)+'0');
	if(value >= 1000)
		USART1_Write(((value/1000)%10)+'0');
	if(value >= 100)
		USART1_Write(((value/100)%10)+'0');
	if(value >= 10)
		USART1_Write(((value/10)%10)+'0');
	USART1_Write((value%10)+'0');
}

unsigned char USART1_IsData(void)
{
	if((UCSR1A & (1<<RXC1)))
		return true;
	else
		return false;
}

unsigned char USART1_Read(void)
{
	while(!(UCSR1A & (1<<RXC1)))
		;
	return UDR1;
}